<?php
/**
 * Umlclass-Plugin: Parses uml class blocks
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Marcin Godlewski <godlewski.marcin@op.pl>, some solutions based on Ditaa-Plugin (http://wiki.splitbrain.org/plugin:ditaa)
 * @version    1.0
 */

// must be run within Dokuwiki
if(!defined('DOKU_INC')) die();

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');

class syntax_plugin_umlclass extends DokuWiki_Syntax_Plugin
{
	var $tempdir = "/tmp";	

	function getInfo()
	{
		return array
		(
			'author' => 'Marcin Godlewski',
			'email'  => 'godlewski.marcin@op.pl',
			'date'   => '2009-07-06',
			'name'   => 'Umlclass Plugin',
			'desc'   => 'Parses uml class blocks',
			'url'    => 'http://www.dokuwiki.org/wiki:plugins',
		);
	}

	function getType()
	{
		return 'substition';
	}

	function getPType()
	{
		return 'normal';
	}

	function getSort()
	{
		return 500;
	}

	function connectTo($mode)
	{
		$this->Lexer->addEntryPattern('<uml class>(?=.*?</uml class>)',$mode,'plugin_umlclass');
	}
 
	function postConnect()
	{
		$this->Lexer->addExitPattern('</uml class>','plugin_umlclass');
	}

	function handle($match, $state, $pos, &$handler)
	{
		switch ($state)
		{
			case DOKU_LEXER_ENTER     :return array($state, '');
			case DOKU_LEXER_UNMATCHED :return array($state, $match);
			case DOKU_LEXER_EXIT      :return array($state, '');
		}
		return array();
	}

	function render($mode, &$renderer, $data)
	{
		global $conf;
		global $INFO;

		if($mode == 'xhtml')
		{
				$ret = true;				
				list($state, $match) = $data;
		   	switch ($state)
				{
					case DOKU_LEXER_ENTER:
						break;

					case DOKU_LEXER_UNMATCHED:

						$tempfile = $this->tempdir . '/umlclass_plugin_tmp';

					  	$file = fopen($tempfile.'.java', 'w');
					  	fwrite($file, $match);
						fclose($file);

					  	$md5 = md5_file($tempfile.'.java');

					  	$mediadir = $conf["mediadir"]."/".str_replace(":", "/",$INFO['namespace'] );

						if( !is_dir($mediadir) )
							mkdir($mediadir,0755);

					  	$imagefile = $mediadir.'/umlclass_plugin' . '_' . $md5 . '.png';

					  	if ( !file_exists($imagefile) )
					  	{						   
							$cmd = 'umlgraph ' . $tempfile . ' png';
						   exec($cmd, $output, $error);
						   if ($error != 0)
							{
								$renderer->doc .= '---ERROR CREATING GRAPH---';
								unlink($tempfile.'.java');
								if (file_exists($tempfile.'.png'))
									unlink($tempfile.'.png');
						      return false;
						   }

						   if (file_exists($imagefile))
								unlink($imagefile);

							if ( !copy($tempfile.'.png', $imagefile) )
								return false;
						
						   unlink($tempfile.'.png');
					  	}

					  	unlink($tempfile.'.java');

						$renderer->doc .= $renderer->internalmedia($INFO['namespace'].':umlclass_plugin' . '_' . $md5 . '.png');

					  	break;

					case DOKU_LEXER_EXIT:
						break;
				}
				return true;
		}
		return false;
	}

}
?>
